/** @file pwt_adjlist.h Adjacency list


 * This class implements an adjacency list data structure representing
 * a given generator. It can be described simply as an array of linked 
 * lists. For each state i, the array contains a list of its successors
 * on position adj_list[i] - 1 (assuming the states are numbered 
 * consecutively).
 *
 * This data structure is used to simplify the course of a few graph 
 * algorithms, namely a modified version of the DFS algorithm which
 * searches for a cycle in the graph.
 *
 * The class also provides an easy way to visualize a generator in
 * the console.
 */

#ifndef FAUDES_PWT_ADJLIST_H
#define FAUDES_PWT_ADJLIST_H

#pragma once
#include <iostream>
#include <list>
#include "corefaudes.h"
#include "pwt_adjmatrix.h"
#include "pwt_isacyc.h"

namespace faudes {
 
class AdjacencyList {


public:

	/**
	 * The number of states of the represented generator determines 
	 * the size of the array. */
	int size;
	/**
	 * The value of this variable is 'true' only if the states are 		 * numbered consecutively. */
	bool isCons;
	/** 
	 * The array of linked lists representing a list of successors
	 * for each state. */
	std::list<Idx> * adj_list;
	/**
	 * This array stores an alternative index for each state
	 * if the states aren't numbered consecutively. Otherwise
	 * it isn't used. */
	Idx * idxArray;	

	/** Construct from a generator.*/
	AdjacencyList(Generator g);
	
	/** Construct from an adjacency matrix representing a generator.*/
	AdjacencyList(AdjacencyMatrix m);	
	
	/** Destructor.*/
	~AdjacencyList();
	
	/** Write the adjacency list into a console (consecutive 		 * version).*/
	void WriteCons();
	
	/** Write the adjacency list into a console (non-consecutive 		 * version).*/
	void WriteNonCons();
		
	/**
	 * This method calls one of the methods WriteCons() or 
	 * WriteNonCons(), depending on the value of isCons. */	
	void Write();
	
	/**
	 * This function verifies whether the generator represented by
	 * this AdjacencyList object contains a cycle by running the DFS
	 * algorithm on each state, remembering the states that have 
	 * already been visited. If a state is visited for the second
	 * time in one session, a cycle has been detected.
	 *
	 * @param i
	 * 	Index of the first state the DFS is currently visiting
	 *
	 * @pram visitedArray
	 *	Array of bool values that holds information about
	 *      each state: the value is 'true' if it has been visited 
	 *	at least once, otherwise 'false'.
	 *
	 * @param recArray
	 *	Array of bool values that holds information about
	 *	each state: 'true' if it has been visited during 
	 *	the current run of the algorithm, otherwise 'false'.
	 *
	 * @return
	 * 'true' if a cycle containing state i has been detected,
	 * otherwise 'false'
	 */
	bool HasCycleRec(int i, bool * visitedArray, bool * recArray);
	
	/**
	 * This method runs the HasCycleRec() method on each state of
	 * the generator.
	 *
	 * If no cycle has been detected, the metod returns a 'true'
	 * value. Otherwise it returns a 'false' value immediately after
	 * finding a cycle.
	 */
	
	bool IsAcyclic();
	/**
	 * For a non-consecutive set of states, this method returns
	 * the alternative (consecutive) value for each index. Re-
	 * numbering the states in a consecutive way (starting with 1) 
	 * allows the DFS to be run with no errors. In a consecutively 
	 * numbered graph, the method has no use.
	 *
	 * @param index
	 * The original index of the input state.
	 *
	 * @return
	 * The new index of the input state.
	 */
	
	int FindIdx(Idx index);	
}; 
} //namespace faudes
#endif
