/** @file pwt_adjlistud.h Adjacency list (undirected graph version)
 *
 * This class is very similar to the AdjacencyList data structure, with
 * the difference that it allows us to see a directed graph as if it were
 * undirected.
 *
 * The data structure is also implemented as an array of linked lists.
 * For each state, there's a list provided of its successors as well as
 * predecessors. The union of successor and predecessor states of one state
 * in directed graph G is equivalent to the set of all neighbors of 
 * the symmetrization of G.
 *
 * This class is mainly utilized to simplify the search for weakly 
 * connected components of a graph representing a generator. 
 */
 
#ifndef FAUDES_PWT_ADJLISTUD_H
#define FAUDES_PWT_ADJLISTUD_H

#pragma once
#include <iostream>
#include <list>
#include "corefaudes.h"
#include "pwt_isacyc.h"

namespace faudes {


 
class AdjacencyListUndirected {

public:
	/**
	 * The size of the array is equal to the number of states of the 
	 * relevant generator.*/
	int size;
	/**
	 * The value of this variable is 'true' only if the states are 		 * numbered consecutively. */	
	bool isCons;
	/**
	 * The array of linked lists representing a list of successors
	 * as well as precedessors of each state. */	
	std::list<Idx> * adj_list;
	/** 
	 * This array stores an alternative index for each state
	 * if the states aren't numbered consecutively. Otherwise
	 * it isn't used. */
	Idx * idxArray;
	
	/** Constructs from a generator.*/
	AdjacencyListUndirected(Generator g);
	
	/**
	 * Constructs from a generator and a two-element subset B
	 * of its alphabet. This constructor reduces the set
	 * of transitions from the input generator to only include
	 * those labeled by events from B.
	 *
	 * @param Generator g
	 * Input generator
	 *
	 * @param Idx ev1
	 * Index of the first event present in the subset B.
	 *
	 * @param Idx ev2
	 * Index of the second event present in the subset B.
	 */
	AdjacencyListUndirected(Generator g, Idx ev1, Idx ev2);
	
	/** Destructor.*/
	~AdjacencyListUndirected();
	
	/**
	 * Write the adjacency list into a console (consecutive 		 * version).*/	
	void WriteCons();
	
	/** 
	 * Write the adjacency list into a console (non-consecutive 		 * version).*/	
	void WriteNonCons();
	
	/**
	 * This method calls one of the methods WriteCons() or 
	 * WriteNonCons(), depending on the value of isCons. */	
	void Write();
	
	/**
	 * For a non-consecutive set of states, this method returns
	 * the alternative (consecutive) value for each index. Re-
	 * numbering the states in a consecutive way (starting with 1) 
	 * allows the DFS to be run with no errors. In a consecutively 
	 * numbered graph, the method has no use.
	 *
	 * @param index
	 * The original index of the input state.
	 *
	 * @return
	 * The new index of the input state.
	 */
	int FindIdx(Idx index);
	
	/**
	 * This recursive method searches for a connected component of
	 * the graph containing the given index. It searches the generator
	 * by running the DFS algorithm. The recursion stops when each
	 * successor of the input state (represented by its index) has
	 * been visited.
	 *
	 * @param index
	 * Index of the state where the DFS algorithm starts its
	 * current course
	 *
	 * @param visitedArray
	 * Array of bool values indicating if a state on the i-th position
	 * has been visited by the DFS algorithm
	 *
	 * @return
	 * An array of bool values indicating which states are in the
	 * same connected component as the input index
	 **/
	
	bool * ConnectedComponentRec(Idx index, bool * visitedArray);
	
	/**
	 * This method runs the ConnectedComponentRec() method and
	 * then converts the output visitedArray into a set of states.
	 * If the bool value on the i-th position of this array is 'true', 
	 * the state on the i-th position belongs to the same connected
	 * component as the state represented by the input index.
	 *
	 * @param index
	 * State represented by its index in the StateSet of the relevant
	 * generator
	 *
	 * @return 
	 * StateSet of all states present in the weakly connected
	 * component of the graph containing the input index
	 */
	
	StateSet ConnectedComponent(Idx index);
	
	/**
	 * This method calls the ConnectedComponent() method with either
	 * the input index (if the value of isCons is 'true') or with its
	 * alternative index stored in idxArray (if the states are not 
	 * numbered consecutively). This distinction ensures the correct
	 * result for each case.
	 *
	 * @param index
	 * State represented by its index in the StateSet of the relevant
	 * generator
	 *
	 * @return
	 * StateSet of all states present in the weakly connected
	 * component of the graph containing the input index	 
	 */	
	StateSet MakeCC(Idx index);
	
	/**
	 * This method displays the indices of all states present 
	 * in the weakly connected component of the state represented
	 * by the input index. 
	 * 
	 * @param index
	 * State represented by its index in the StateSet of the relevant
	 * generator
	 */	
	void WriteCC(Idx index);
}; 
} //namespace faudes
#endif
