/** @file pwt_adjmatrix.h Adjacency matrix
 *
 * This class implements the adjacency matrix data structure 
 * representing a given generator. It can be described simply
 * as a two-dimensional array of indices. For a state i and a 
 * letter j, the element on the (i, j)-th position of the matrix
 * is equal to the successor of state i labeled by letter j.
 * If such transition doesn't exist, the value on this position
 * is set to be 0.
 *
 * The class is used by the AdjacencyList class to extract data
 * from. It also provides a way to visualize a generator in the 
 * console. 
 */
#ifndef FAUDES_PWT_ADJMATRIX_H
#define FAUDES_PWT_ADJMATRIX_H

#pragma once
#include <iostream>
#include <list>
#include "corefaudes.h"
#include "pwt_isacyc.h"

namespace faudes {

class AdjacencyMatrix {

public:

	/** The height of the matrix is equal to the number of states.*/
	int height;
	/** The width of the matrix is equal to the size of the alphabet.*/
	int width;
	/** This value shows whether the input generator contains 
	 * consecutively numbered states.*/
	bool isCons;
	/** The actual matrix: a two-dimensonal array of indices.*/
	Idx ** adj_matrix;
	/**
	 * The array providing an alternative index to each state (only
	 * used if the states are not numbered consecutively). */
	Idx * state_array;	

	/** Constructs from a generator.*/
	AdjacencyMatrix(Generator g);
	
	/**
	 * Write the adjacency matrix into a console (non-consecutive 
	 * version).*/
	void WriteCons();
	
	/**
	 * Write the adjacency matrix into a console (non-consecutive
	 * version).*/
	void WriteNonCons();
	
	/**
	 * This method calls one of the methods WriteCons() or 
	 * WriteNonCons(), depending on the value of isCons. */	
	void Write();
	
	/**
	 * For a non-consecutive set of states, this method returns
	 * the alternative (consecutive) value for each index. Re-numbering 
	 * the states is an essential step to make the DFS algorithm run 
	 * correctly on an adjacency list constructed from the matrix.
	 *
	 * @param index
	 * The original index of the input state.
	 *
	 * @return
	 * The new index of the input state.
	 */	
	int FindIdx(Idx index);	
}; 
}//namespace faudes
#endif
