/** @file pwt_checkinput.h Checking properties of the input generator
 *
 * The purpose of this set of functions is to convert the input generator
 * G into a deterministic, minimal formal automaton which accepts the 
 * same language as G.
 * 
 * Generator G is deterministic iff it contains only one initial state
 * and at most one transition in the form delta(q, a) for each state
 * q and each event a from G. This property is checked by the 
 * faudes::IsDeterministic(Generator g) function. If the generator isn't
 * deterministic, it is converted by the faudes::Deterministic(Generator 
 * g1, Generator g2) function. 
 *
 * Generator G is converted into its minimal version by removing 
 * inaccessible states and unifying mutually equivalent states. This is
 * done with the faudes::StateMin(Generator g1, Generator g2) function
 * which implements Hopcroft's algorithm.
 *
 * Generator G is considered a formal automaton if it's deterministic
 * and if it contains  a transition in the form delta(q, a) for each
 * state q and each event a. To convert a generator into a formal automaton,
 * we utilize the function faudes::Automaton(Generator g1, Generator g2).
 * */
#ifndef FAUDES_PWT_CHECKINPUT_H
#define FAUDES_PWT_CHECKINPUT_H

#pragma once
#include <iostream>
#include "corefaudes.h"

namespace faudes {

/** 
 * This function verifies if the input generator is a formal automaton.
 * This condition is met if the transition function is total, i.e. 
 * if the number of transitions of g is equal to the product |Q| * |A|
 * where Q is the StateSet of g and A is the EventSet of g. The input 
 * generator is expected to be deterministic.
 *
 * @param g
 * input generator
 *
 * @return
 * 'true' if the input generator is an automaton, otherwise 'false'
 *
 * @ingroup PwtestabilityPlugin 
 */

bool IsAutomaton(Generator g);

/**
 * This function returns the deterministic, minimal version of the
 * input generator that is also guaranteed to be a formal automaton.
 *
 * @param g
 * input generator
 *
 * @return
 * deterministic, minimal version of g with a total transition function
 *
 * @ingroup PwtestabilityPlugin 
 */

Generator CheckInputProps(Generator g);

} //namespace faudes
#endif
