/** @file pwt_isacyc.h Functions searching for cycles in a generator
 *
 * This set of functions is used to search for a cycle in a graph
 * representing a generator. It works for both a consecutively and 
 * non-consecutively numbered StateSet.
 * This approach is faster than the ones utilizing the help of
 * additional data structures (AdjacencyMatrix, AdjacencyList), therefore
 * it is recommended to utilize these functions in programs where 
 * short duration is a priority. On the other hand, it doesn't provide
 * any visual illustration for users who might be interested in
 * inspecting the graph which represents the input generator.
 **/
#ifndef FAUDES_PWT_ISACYC_H
#define FAUDES_PWT_ISACYC_H

#pragma once
#include <iostream>
#include <list>
#include "corefaudes.h"



namespace faudes {

/**
 * This function checks whether the states of a given generator are
 * numbered consecutively (i.e. 1, 2, 3, ..., n for a generator 
 * containing n states).
 * This is true if the size of the StateSet is equal to the value of
 * the last index.
 *
 * @param g
 * input generator
 *
 * @return
 * 'true'/'false' if the states of the generator are/aren't numbered
 * consecutively
 *
 * @ingroup PwtestabilityPlugin 
 **/

bool IsConsecutive(Generator g);

/**
 * This function creates an array of indices representing all states
 * of an input generator. Converting a StateSet object into an array 
 * of indices facilitates the course of the DFS algorithm which 
 * primarily works with arrays.
 *
 * @param g
 * input generator
 *
 * @return
 * array of indices representing each state
 *
 * @ingroup PwtestabilityPlugin 
 **/

Idx * MakeIndexArray(Generator g);

 /**
  * This function searches the array created by the MakeIndexArray()
  * function. Its purpose is to find a new index assigned to a state
  * in a non-consecutively numbered graph. The new value is equal to
  * i where indexArray[i] == index.
  *
  * @param g
  * input generator
  *
  * @param indexArray
  * array of indices representing the StateSet of g
  *
  * @param index
  * original index of some state from g
  *
  * @return
  * the new index of a state which will be used by the DFS algorithm
  **/

int FindIndex(Generator g, Idx * indexArray, Idx index);

/**
 * This recursive function verifies whether there is a cycle 
 * containing the input state i. It utilizes a modified version
 * of the DFS algorithm that keeps track of the visit history of
 * each state. The function is specifically used for a StateSet that
 * is numbered non-consecutively: alternative, consecutively numbered
 * indices are kept in the indexArray. The algorithm stops and returns
 * the 'true' value if some state has been visited for the second time.
 * If no cycle is present, the function returns the 'false' value after
 * visiting each successor of the input state.
 * 
 * @param g
 * input generator
 *
 * @param i
 * index of the first state the DFS is currently visiting
 *
 * @param visitedArray
 * array of bool values that holds information about
 * each state: the value is 'true' if it has been visited 
 * at least once, otherwise 'false'
 *
 * @param recArray
 * Array of bool values that holds information about
 * each state: 'true' if it has been visited during 
 * the current run of the algorithm, otherwise 'false'
 *
 * @param indexArray
 * array of original, non-consecutive indices for each state
 *
 * @return
 * 'true' if a cycle containing state i has been detected,
 * otherwise 'false'
 */

bool HasCycleNonCons(Generator g, int i, bool visitedArray[], bool recArray[], Idx indexArray[]);
  
/**
 * This function verifies whether the input generator with
 * non-consecutively numbered states is acyclic by calling
 * the HasCycleNonCons() function. If the function returns
 * a 'false' value, the graph is acyclic (and vice versa).
 *
 * @param g
 * input generator
 *
 * @return
 * 'true' if the generator is acyclic, 'false' if it contains a cycle
 */
  
bool IsAcyclicNonCons(Generator g);

/**
 * This recursive function verifies whether there is a cycle 
 * containing the input state i. It works on the same principle as
 * the function HasCycleNonCons(), with the difference that this
 * function always has a consecutively-numbered input StateSet.
 * It utilizes a modified version of the DFS algorithm that keeps
 * track of the visit history of each state.
 * The algorithm stops and returns the 'true' value if some state has been 
 * visited for the second time. That means a cycle has been discovered.
 * If no cycle is present, the function returns the 'false' value after
 * visiting each successor of the input state.
 * 
 * @param g
 * input generator
 *
 * @param i
 * index of the first state the DFS is currently visiting
 *
 * @param visitedArray
 * array of bool values that holds information about
 * each state: the value is 'true' if it has been visited 
 * at least once, otherwise 'false'
 *
 * @param recArray
 * Array of bool values that holds information about
 * each state: 'true' if it has been visited during 
 * the current run of the algorithm, otherwise 'false'
 *
 * @return
 * 'true' if a cycle containing state i has been detected,
 * otherwise 'false'
 */

bool HasCycleCons(Generator g, int i, bool visitedArray[], bool recArray[]);
  
/**
 * This function works in a similar way as the IsAcyclicNonCons() 
 * function, however its input StateSet is consecutively numbered.
 * The function verifies whether the input generator with
 * non-consecutively numbered states is acyclic by calling
 * the HasCycleNonCons() function. If the function returns a 'false' 
 * value, the graph is acyclic (and vice versa).
 *
 * @param g
 * input generator
 *
 * @return
 * 'true' if the generator is acyclic, 'false' if it contains a cycle
 */
  
bool IsAcyclicCons(Generator g);

/**
 * This function calls either the IsAcyclicCons() or IsAcyclicNonCons()
 * function, depending on the result of the IsCons() function.
 *
 * @param g
 * input generator
 *
 * @return
 * 'true' if the generator is acyclic, 'false' if it contains a cycle
 *
 * @ingroup PwtestabilityPlugin 
 */

bool IsAcyclic(Generator g);

} //namespace faudes
#endif
