/** @file pwt_klimakuncpolak.h Piecewise testability (Klíma, Kunc and Polák's algorithm)
 *
 * This set of functions implements the algorithm coined by O.Klíma,
 * M.Kunc and L.Polák that verifies piecewise testability of a regular
 * language accepted by generator g.
 *
 * The first step is to check if the oriented graph representing
 * the input generator g is acyclic. This is done with the help of the
 * AdjacencyList data structure. If a cycle is found, the language
 * accepted by g isn't piecewise testable and the program stops.
 *
 * The second step is to check for every state from G and for every
 * two-element subset B of the alphabet Sigma if each weakly connected
 * component of B|G (graph containing all states from G and only those
 * transitions from G labeled by letters from B) contains only one B-
 * absorbing state. If this condition is not met for every state and
 * every two-element subset B, the language accepted by G isn't
 * piecewise testable.
 **/
 
#ifndef FAUDES_PWT_KLIMAKUNCPOLAK_H
#define FAUDES_PWT_KLIMAKUNCPOLAK_H

#pragma once
#include <iostream>
#include <list>
#include "corefaudes.h"
#include "pwt_checkinput.h"
#include "pwt_adjlist.h"
#include "pwt_adjlistud.h"

namespace faudes {

/**
 * This function verifies if the state represented by index st is 
 * B-absorbing for the two-element alphabet B which consists of
 * events ev1 and ev2.
 *
 * For EventSet B, a state q is B-absorbing if for every letter x from
 * B, the transition delta(q, x) is a loop.
 *
 * @param g
 * input generator
 *
 * @param st
 * index representing some state of generator g
 *
 * @param ev1
 * index representing a letter from B (different from ev2)
 *
 * @param ev2
 * index representing a letter from B (different from ev1)
 *
 * @return
 * 'true' if both transitions delta(st, ev1) and delta(st, ev2) are
 * loops, otherwise 'false'
 *
 * @ingroup PwtestabilityPlugin 
 **/

bool IsBAbsorbing(Generator g, Idx st, Idx ev1, Idx ev2);

/**
 * This function verifies if the weakly connected component of the input
 * graph g (with a TransSet restricted by B) represented by StateSet set
 * contains at most one B-absorbing state for alphabet B = {ev1, ev2}
 *
 * @param g
 * input generator
 *
 * @param set
 * StateSet representing some weakly connected component of g (with
 * a TransSet restricted by B)
 *
 * @param ev1
 * index representing some letter from generator g (part of B)
 *
 * @param ev2
 * index representing some letter from generator g (part of B)
 *
 * @return
 * 'true' if set contains at most one B-absorbing state where 
 * B = {ev1, ev2}
 *
 * @ingroup PwtestabilityPlugin
 **/

bool HasOneBAbsSt(Generator g, StateSet set, Idx ev1, Idx ev2);

/**
 * This function creates a list of all weakly connected components of
 * the graph B|G where B|G represents generator g with TransSet reduced
 * to only include transitions labeled by letters from B, B = {ev1, ev2}.
 *
 * @param g
 * input generator
 *
 * @param ev1
 * index representing an event from g, ev1 is different from ev2
 *
 * @param ev2
 * index representing an event from g, ev2 is different from ev1
 *
 * @return
 * a list of StateSet objects, each StateSet representing one weakly
 * connected component of g
 *
 * @ingroup PwtestabilityPlugin 
 **/

std::list<StateSet> MakeStateSetList(Generator g, Idx ev1, Idx ev2);

/**
 * This function writes the list of connected components created by
 * MakeStateSetList() into the console. It isn't an essential part
 * of the algorithm but it offers helpful visual representation of
 * all components of g.
 *
 * @param g
 * input generator
 *
 * @param ev1
 * index representing an event from g, ev1 is different from ev2
 *
 * @param ev2
 * index representing an event from g, ev2 is different from ev1
 *
 * @ingroup PwtestabilityPlugin
 **/

void WriteStateSetList(Generator g, Idx ev1, Idx ev2);

/**
 * This function verifies if the regular language accepted by the
 * input generator g is piecewise testable. It implements the algorithm
 * described above.
 *
 * @param g
 * input generator
 *
 * @return
 * 'true' if the language accepted by g is piecewise testable, otherwise 
 * 'false'
 *
 * @ingroup PwtestabilityPlugin
 **/

bool IsPiecewiseTestable_KlimaKuncPolak(Generator g);

} //namespace faudes
#endif
