/** @file pwt_klimapolak.h Piecewise testability (Klíma and Polák's algorithm)
 *
 * This file implements the algorithm coined by O.Klíma and L.Polák that
 * verifies piecewise testability of a regular language accepted by
 * generator g.
 *
 * The first step is to check if the oriented graph representing
 * the input generator is acyclic. This is done with the help of the
 * AdjacencyMatrix and AdjacencyList data structures. If a cycle is
 * found, the language accepted by g isn't piecewise testable and the
 * program stops.
 *
 * In the second step, the algorithm calculates for each state q from g
 * and for every pair of letters a, b from g the following expressions:
 *
 * -> q, qa, qaa, qaab, ..., qa(ab)^m;
 *
 * -> q, qb, qba, qbab, ... qb(ab)^m
 *
 * where m is the number of states of g. If for any trio q, a, b
 * the expressions qa(ab)^m, qb(ab)^m aren't equal, the language isn't
 * piecewise testable and the program returns a 'false' value.
 */
 
#ifndef FAUDES_PWT_KLIMAPOLAK_H
#define FAUDES_PWT_KLIMAPOLAK_H

#pragma once
#include <iostream>
#include <list>
#include "corefaudes.h"
#include "pwt_checkinput.h"
#include "pwt_adjmatrix.h"
#include "pwt_adjlist.h"

namespace faudes {

/**
 * This function verifies whether the regular language accepted by the
 * input generator is piecewise testable. It implements the algorithm
 * described above. 
 *
 * @param g
 * input generator
 *
 * @return
 * 'true' if the language accepted by g is piecewise testable, otherwise 
 * 'false'
 *
 * @ingroup PwtestabilityPlugin
 **/

bool IsPiecewiseTestable_KlimaPolak(Generator g);

} //namespace faudes
#endif
