/** @file pwt_trahtman.h Piecewise testability (Trahtman's algorithm)
 *
 * This set of functions implements the algorithm coined by
 * A.N.Trahtman that verifies piecewise testability of a regular
 * language accepted by generator g.
 *
 * The first step is to check if the oriented graph G representing
 * the input generator g is acyclic. This is done by running a modified
 * version of the DFS algorithm on the generator. If a cycle is found,
 * the language accepted by g isn't piecewise testable and the program 
 * stops.
 *
 * The second step is to compute the following for each state p
 * from g:
 *
 * - the stabilizer Sigma(p), which is a set of letters x such that any
 * transition delta(p, x) is a loop
 *
 * - the graph Sigma(p)|G containing all states from g and only
 * the transitions from g labeled by letters from Sigma(p)
 *
 * - non-oriented copy N of Sigma(p)|G
 *
 * - the component C of N containing state p
 *
 * - for any node r from C, r!=p, the algorithm checks if r has any 
 * outgoing edges that aren't loops. If no such edges can be found, the
 * language isn't piecewise testable and the algorithm returns the 'false'
 * value.
 **/
#ifndef FAUDES_PWT_TRAHTMAN_H
#define FAUDES_PWT_TRAHTMAN_H

#pragma once
#include <iostream>
#include <list>
#include "corefaudes.h"
#include "pwt_checkinput.h"
#include "pwt_isacyc.h"
#include "pwt_adjlist.h"
#include "pwt_adjlistud.h"

namespace faudes {

/**
 * This function verifies if the input transition t is a loop, i.e. if
 * its states X1 and X2 are equal.
 *
 * @param t
 * input transition
 *
 * @return
 * 'true' if states X1, X2 of t are equal, otherwise 'false'
 *
 * @ingroup PwtestabilityPlugin 
 **/

bool isLoop(Transition t);

/**
 * This function verifies if generator g has no outgoing edges from the
 * state represented by index i or if all edges outgoing from i are
 * loops.
 *
 * @param g
 * input generator
 *
 * @param i
 * index representing the input state from g
 *
 * @return
 * 'true' if there are no outgoing edges from i in g that aren't loops,
 * otherwise 'false'
 *
 * @ingroup PwtestabilityPlugin
 **/

bool NoOutgoingEdges(Generator g, Idx i);

/**
 * This function computes the stabilizer of state indexed by i in g.
 * In generator g, a stabilizer of state i is a set of all letters
 * that label loops from i to i.
 *
 * @param g
 * input graph
 *
 * @param i
 * index representing some state of g
 *
 * @return
 * EventSet including all events that label loops from i to i
 *
 * @ingroup PwtestabilityPlugin 
 **/

EventSet MakeStabilizer(Generator g, Idx i);

/** This function calls the MakeStabilizer() function with arguments
 * g and i and then creates a copy of g which includes all states
 * from g and only the letters from g that belong to the stabilizer
 * of state i.
 *
 * @param g
 * input generator
 *
 * @param i
 * index representing some state of g
 *
 * @return
 * a copy of g with EventSet (and TransSet) restricted to only include
 * letters from the stabilizer of i
 *
 * @ingroup PwtestabilityPlugin 
 */

Generator MakeStabilizerGraph(Generator g, Idx i);

/**
 * This function verifies if the regular language accepted by the
 * input generator g is piecewise testable. It implements the algorithm
 * described above.
 *
 * @param g
 * input generator
 *
 * @return
 * 'true' if the language accepted by g is piecewise testable, otherwise 
 * 'false'
 *
 * @ingroup PwtestabilityPlugin
 */

bool IsPiecewiseTestable_Trahtman(Generator g);

} //namespace faudes
#endif
